<?php

declare(strict_types=1);

namespace Drupal\wse\Controller;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\diff\Controller\NodeRevisionController;
use Drupal\node\NodeInterface;
use Drupal\workspaces\Plugin\Validation\Constraint\EntityWorkspaceConflictConstraint;
use Drupal\workspaces\WorkspaceManagerInterface;
use Drupal\wse\Diff\Form\WseRevisionOverviewForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Overrides the node revision controller for the Diff module.
 */
class WseDiffNodeRevisionController extends NodeRevisionController {

  /**
   * The workspace manager.
   */
  protected WorkspaceManagerInterface $workspaceManager;

  /**
   * The typed data manager.
   */
  protected TypedDataManagerInterface $typedDataManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    $instance = parent::create($container);
    $instance->workspaceManager = $container->get('workspaces.manager');
    $instance->typedDataManager = $container->get('typed_data_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function revisionOverview(NodeInterface $node): array {
    if (!$node->access('view')) {
      throw new AccessDeniedHttpException();
    }
    $build = $this->formBuilder()->getForm(WseRevisionOverviewForm::class, $node);

    if ($this->workspaceManager->hasActiveWorkspace()) {
      $build['node_revisions_table']['#empty'] = $this->t('There are no revisions yet in this workspace.');
      $build['#cache']['contexts'][] = 'workspace';
    }

    // Run the workspace conflict validation constraint when the entity form is
    // being built, so we can "disable" it early and display a message to the
    // user, instead of allowing them to enter data that can never be saved.
    foreach ($node->getTypedData()->getConstraints() as $constraint) {
      if ($constraint instanceof EntityWorkspaceConflictConstraint) {
        $violations = $this->typedDataManager->getValidator()->validate(
          $node->getTypedData(),
          $constraint
        );
        if ($violations->count()) {
          $build['node_revisions_table']['#access'] = FALSE;
          $build['submit']['#access'] = FALSE;
          $build['workspace_conflict']['#markup'] = $violations->get(0)->getMessage();
        }
      }
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getRevisionIds(EntityStorageInterface $storage, $entity_id): array {
    $query = $storage->getQuery()
      ->allRevisions()
      ->condition($storage->getEntityType()->getKey('id'), $entity_id)
      ->accessCheck(FALSE);

    // When a workspace is active, filter the revisions to those created in that
    // workspace.
    if ($this->workspaceManager->hasActiveWorkspace()) {
      $active_workspace_id = $this->workspaceManager->getActiveWorkspace()->id();
      $query->condition($storage->getEntityType()->getRevisionMetadataKey('workspace'), $active_workspace_id);
    }
    else {
      // Otherwise show only revisions that were not created in a workspace.
      $query->notExists($storage->getEntityType()->getRevisionMetadataKey('workspace'));
    }

    $result = $query->execute();
    $result_array = array_keys($result);
    sort($result_array);
    return $result_array;
  }

}
